/*
 * dtio.js - date and time input/output functions.
 *
 * Verlof Web application
 *
 * Copyright (c) 2002, by Leiden University.
 * verlof is free software.  See the file Copying for copying permission.
 *
 * $Revision: 1.2 $
 * $Date: 2003-01-29 09:20:42+01 $
 * $Source: \\\\fwneld\\msdos\\RCS\\K\\fsc\\prj\\verlof\\src\\lib\\dtio.js,v $
 * %Author: M.J. Moene %
 */

/*
 * $Header: \\\\fwneld\\msdos\\RCS\\K\\fsc\\prj\\verlof\\src\\lib\\dtio.js,v 1.2 2003-01-29 09:20:42+01 moene Exp $
 *
 * $Log: dtio.js,v $
 * Revision 1.2  2003-01-29 09:20:42+01  moene
 * <>
 *
 * Revision 1.1  2002/05/13 09:17:11  moene
 * Initial revision
 *
 */

/*
------------------------------------------------------------------------------
strftime                                                       public function
------------------------------------------------------------------------------

Function        format date and time into string according to format string.

Syntax          function strftime(format: String, dt: Date)

Remarks         strftime formats date and time information from dt into
                string strng according to the format string.

                Ordinary characters are copied into strng. Each %c is
                replaced as described below, using values appropriate
                for the local environment.

                strftime returns the string built.

                %a  abbreviated weekday name.
                %A  full weekday name.
                %b  abbreviated month name.
                %B  full month name.
                %c  local date and time representation (01-02-1996 12:34:56).
                %d  day of the month                   (01-31).
                %H  hour (24-hour clock)               (00-23).
              [ %I  hour (12-hour clock)               (01-12).          ]
              [ %j  day of the year                   (001-366).         ]
                %m  month                              (01-12).
                %M  minute                             (00-59).
              [ %p  local equivalent of AM or PM       (VM, NM, AM, PM). ]
                %S  second                             (00-59).
              [ %U  weeknumber of the year             (00-53),          ]
                    monday as 1st day of week !!
                %w  weekday                            ( 1-7 ),
                    monday is 1, sunday is 7 !!
              [ %W  weeknumber of the year             (00-53),          ]
                    monday as 1st day of week
                %x  local date representation          (01-02-1996).
                %X  local time representation          (12:34:56).
                %y  year without century               (00-99).
                %Y  year with century.
                %Z  timezone (not implemented)         ( (Timezone) ) !!
                %%  %.

Note            this function is analogous to the C runtime-library function
                of the same name, but not completely compatible with it. See !!.

Arguments       format: format string,
                dt    : date and time information.

Return value    string built.


------------------------------------------------------------------------------
scanDate                                                       public function
------------------------------------------------------------------------------

Function        initialize Date object's date or time from string.

Syntax          function scanDate(strng: String[, result: scanDateResult] ): Date

Remarks         scanDate interpretes the specified string as a date specification
                and returns the result in a Date object. The time part of dt is
                set to 'now'.

                If a valid date could not be determined, scanDate returns null.

                The date specification can have the following formats:

                22 [apr [96]], apr 22 96, 22-4-96, 96-4-22, 220496, -3, friday, (.).

                Type  Template            Example       Description
                day   day                 friday        next friday
                mdy   mon  dd  (yy)?      jan 12 96
                dmy   dd   mon (yy)?      12 jan 96
                dmy   dd ( mm  (yy)? )?   12 1 96
                ymd   yy ( mm  (dd)? )?   96 1 12       yy > 31
                mdy   mm   dd  (yy)?      1 14 96       mm <= 12 and dd > 12
                dmy   ddmmyy(yy)?         120196
                nil   [+-]n               -2            day before yesterday
                nil   (.)?                .             today

                The dashes between the parts of the date specification are
                may be replaced by spaces.

                Values left unspecified are replaced by the values of 'today'.

                If the year is specified as a number smaller than 100, the
                current century is added automatically.

                The name of month or day may be abbreviated. If the specification of
                a dayname or a monthname is ambiguous (within one language), scanDate
                returns null. E.g. 'z' (zaterdag, zondag, Dutch), or 'ju' (jul, jun,
                Dutch and English).

Arguments       strng : string to scan date from,
                dt    : date, time object for result.

Return value    false, true (ok, error).

*/

/*----------------------------------------------------------------------------
** strftime - format date and time.
**----------------------------------------------------------------------------
*/

/*
 * day- and month names.
 */

dayNames     = [ 'zondag', 'maandag', 'dinsdag', 'woensdag', 'donderdag', 'vrijdag', 'zaterdag' ];
dayAbbrs     = [ 'zo', 'ma', 'di', 'wo', 'do', 'vr', 'za' ];

monthNames   = [ 'januari', 'februari', 'maart', 'april', 'mei', 'juni', 'juli', 'augustus', 'september', 'oktober', 'november', 'december' ];
monthAbbrs   = [ 'jan', 'feb', 'mrt', 'apr', 'mei', 'jun', 'jul', 'aug', 'sep', 'okt', 'nov', 'dec' ];

dayNamesEn   = [ 'sunday', 'monday', 'tuesday', 'wednesday', 'thursday', 'friday', 'saturday' ];
dayAbbrsEn   = [ 'sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat' ];

monthNamesEn = [ 'januari', 'februari', 'march', 'april', 'may', 'june', 'july', 'august', 'september', 'october', 'november', 'december' ];
monthAbbrsEn = [ 'jan', 'feb', 'mar', 'apr', 'may', 'jun', 'jul', 'aug', 'sep', 'oct', 'nov', 'dec' ];

/*
 * strftime - format date and time into string according to format string.
 */

function strftime( format, dt)
{
//alert( 'strftime: ' + format );

   var cha;     /* current format character */
   var ndx;     /* index into formatstring */
   var len;     /* length for recursive call */

   /*
    * initialize strings:
    */

   var strng  = "";
   var tmpstr = "";

   /*
    * scan formatstring:
    */

   ndx = -1;
   while ( true )
   {
      /*
       * add the contents of tmpstr to the result:
       */

      strng  = strng + tmpstr;
      tmpstr = "";

      /*
       * characters left to scan from the formatstring?
       */

      if ( ndx < format.length ) ndx++;
      else                       break;

      cha = format.charAt( ndx );

      /*
       * check for conversion specifier:
       */

      if ( cha != "%" )
      {
         tmpstr = cha;
         continue;
      }

      if ( ndx >= format.length )
         break;

      /*
       * dispatch commands:
       */

      ndx++;
      cha = format.charAt( ndx );

      switch (cha) {
         case 'a': tmpstr = dayAbbrs  [ dt.getDay()   ]; break;
         case 'A': tmpstr = dayNames  [ dt.getDay()   ]; break;
         case 'b': tmpstr = monthAbbrs[ dt.getMonth() ]; break;
         case 'B': tmpstr = monthNames[ dt.getMonth() ]; break;

         case 'd': tmpstr = formatNumber( dt.getDate()       , 2 ); break;
         case 'H': tmpstr = formatNumber( dt.getHours()     , 2 ); break;
//         case 'I': { tmpstr = formatNumber( dt.hour12    , 2 );   }
//         case 'j': { tmpstr = formatNumber( dt.dayOfYear , 3 );   }
         case 'm': tmpstr = formatNumber( dt.getMonth() + 1 , 2 ); break;
         case 'M': tmpstr = formatNumber( dt.getMinutes()       , 2 ); break;
         case 'S': tmpstr = formatNumber( dt.getSeconds()       , 2 ); break;
//         case 'U': { tmpstr = formatNumber( dt.weekOfYear, 2 );   }
         case 'w': tmpstr = formatNumber( dt.getDay() , 1 ); break;
//         case 'W': { tmpstr = formatNumber( dt.weekOfYear, 1 );   }

         case 'y': tmpstr = formatNumber( dt.getFullYear(), 4 ).substr( 2, 2); break;
         case 'Y': tmpstr = formatNumber( dt.getFullYear(), 4 ); break;

         case 'c': tmpstr = strftime( '%x %X'   , dt ); break;
         case 'x': tmpstr = strftime( '%d-%m-%Y', dt ); break;
         case 'X': tmpstr = strftime( '%H:%M:%S', dt ); break;

         case 'Z': tmpstr = '(Timezone)'; break;
//         case 'p': { tmpstr = dt.ampmIndicator; }

         default: tmpstr = cha;            /* %% etc. */
      }
   }

   return strng;
}

/*
 * format an integer into a field of width w; optionally suppress leading zero's.
 */

function formatNumber( num, w, suppress )
{
   if ( suppress == null )
      suppress = false;

   var numstr = num.toString();
   var tmpstr = "";

   for ( i = 0; i < w - numstr.length; i++ )
      tmpstr += suppress ? " " : "0";

   return tmpstr + numstr;
}


/*----------------------------------------------------------------------------
** scanDate - obtain date from string.
**----------------------------------------------------------------------------
*/

/*
 * scanDate - return Date object with date initialized from string.
 *
 * 22 [apr [96]], apr 22 96, 22-4-96, 96-4-22, 220496, -3, friday, (.).
 *
 * type  template            example       description
 * day   day                 friday        next friday
 * mdy   mon  dd  (yy)?      jan 12 96
 * dmy   dd   mon (yy)?      12 jan 96
 * dmy   dd ( mm  (yy)? )?   12 1 96
 * ymd   yy ( mm  (dd)? )?   96 1 12       yy > 31
 * mdy   mm   dd  (yy)?      1 14 96       mm <= 12 and dd > 12
 * dmy   ddmmyy(yy)?         120196
 * nil   [+-]n               -2            day before yesterday
 * nil   (.)?                .             today
 */

/*
 * scanDate result to pass back for searchDay() and searchMonth():
 */

function scanDateResult()
{
   this.clear= scanDateClear;

   this.clear();

   return this;
}

function scanDateClear()
{
   this.stat = "";
   this.estr = "";
   this.type = "nil";
   this.date = null;

   return this;
}

function scanDate( str, result )
{
//alert( 'scanDate: ' + str );

   var dt = new Date();

   if ( null == result )
      result = new scanDateResult();

   result.clear();

   f1 = f2 = f3 = -1;
   dy = mt = yr = -1;

   /*
    * strip leading  and trailing space:
    */

   str = str.replace( /^\s*/, '' );
   str = str.replace( /\s*$/, '' );

   /*
    * day                 friday        (next) friday
    * mon ( dd (yy)? )?   jan 12 96
    */

   if ( str.match( /^([a-z]+)(\D+(\d+)(\D+(\d+))?)?/i ) )
   {
//alert( 'day / mon (dd (yy[yy])?)?' );

      f1 = RegExp.$1;
      if ( RegExp.$3 ) f2 = Number( RegExp.$3 );
      if ( RegExp.$5 ) f3 = Number( RegExp.$5 );

      dayResult = new searchResult();
      monResult = new searchResult();

      nday = searchDay  ( f1, dayResult );
      nmon = searchMonth( f1, monResult );

      if ( nday > 0 && nmon > 0 )
      {
         result.stat = "daymonamb";
         result.estr = f1;
         return null;
      }

      /*
       * day specification?
       */

      else if ( nday > 0 )
      {
         result.type = "day";

         if ( nday > 1 )
         {
            result.stat = "dayamb";
            result.estr = f1;
            return null;
         }

         f1 = dayResult.atr;            // set date
      }

      /*
       * month specification?
       */

      else if ( nmon  > 0 )
      {
         result.type = "mdy";

         if ( nmon > 1 )
         {
            result.stat = "monamb";
            result.estr = f1;
            return null;
         }
         else
         {
            f1 = monResult.atr;            // set month
         }
      }
      else
      {
         result.stat = "daymonnon"
         result.estr = f1;
         return null;                 // cannot determine date
      }
   }

   /*
    * dd  mon  (yy)?   12 feb 2002
    */

   else if ( str.match( /^(\d+)\W+([a-z]+)(\D+(\d+))?/i ) )
   {
//alert( 'dd mon (yy[yy])' );

      f1 = Number( RegExp.$1 );
      if ( RegExp.$2 ) f2 = RegExp.$2;
      if ( RegExp.$4 ) f3 = Number( RegExp.$4 );

      result.type = "dmy";

      monResult = new searchResult();

      /*
       * month specification?
       */

      if ( f2 && ( nmon = searchMonth( f2, monResult ) ) > 0 )
      {
         if ( nmon > 1 )
         {
            result.stat = "monamb";
            result.estr = f2;
            return null;
         }
         else
         {
            f2 = monResult.atr;            // set month
         }
      }
      else
      {
         result.stat = "monval";
         result.estr = f2;
         return null;
      }
   }

   /*
    * dd ( mm  (yy)? )?   12 1 96
    * yy ( mm  (dd)? )?   96 1 12    yy > 31
    * mm   dd  (yy)?      1 14 96    mm <= 12 and dd > 12
    * ddmmyy(yy)?         120196
    */

   else if ( str.match( /^(\d+)(\W+(\d+)(\W+(\d+))?)?/ ) )
   {
//alert( 'dd( mm( yy[yy])) / yy[yy]( mm( dd))' );

      f1 = Number( RegExp.$1 );
      if ( RegExp.$3 ) f2 = Number( RegExp.$3 );
      if ( RegExp.$5 ) f3 = Number( RegExp.$5 );

      result.type = "dmy";

      if ( f1 <= 12 && f2 && f2 > 12    ) result.type = "mdy";
      if (                   f1 > 31    ) result.type = "ymd";
      if (                   f1 > 10000 )
      {
         result.type = "dmy";

         f1 = Number( RegExp.$1.substr( 0, 2 ) );
         f2 = Number( RegExp.$1.substr( 2, 2 ) );
         f3 = Number( RegExp.$1.substr( 4    ) );
      }
//alert( result.type + '/' + f1 + '/' + f2 + '/' + f3 );
   }

   /*
    * +1: tomorrow,
    */

   else if ( str.match( /^[+](\d+)/ ) )
   {
//alert( '+dd' );
      result.type = "ymd";

      dt.setDate( dt.getDate() + Number( RegExp.$1 ) );
      f1 = dt.getFullYear();
      f2 = dt.getMonth() + 1;
      f3 = dt.getDate();
   }

   /*
    * -2: day before yesterday,
    */

   else if ( str.match( /^[-](\d+)/ ) )
   {
//alert( '-dd' );
      result.type = "ymd";

      dt.setDate( dt.getDate() - Number( RegExp.$1 ) );
      f1 = dt.getFullYear();
      f2 = dt.getMonth() + 1;
      f3 = dt.getDate();
   }

   /*
    * .: today (also empty string).
    */

   else if ( str.match( /\./ ) )
   {
//alert( '.' );
   }

   /*
    * determine year, month and day values:
    */

   switch( result.type )
   {
      case "nil": yr = f3; mt = f2; dy = f1; break;
      case "ymd": yr = f1; mt = f2; dy = f3; break;
      case "mdy": yr = f3; mt = f1; dy = f2; break;
      case "dmy": yr = f3; mt = f2; dy = f1; break;
      case "day": dt.setDate( dt.getDate() + 7 - ( 7 +  dt.getDay() - f1 ) % 7 ); break;
   }

   /*
    * substitute defaults for non-specified values:
    */
//alert( 'dy/mt/yr: ' + dy + mt + yr );

   if ( dy < 0 ) dy = dt.getDate ();
   if ( mt < 0 ) mt = dt.getMonth() + 1;
   if ( yr < 0 ) yr = dt.getFullYear();

   /*
    * add century, if necessary:
    */

   if ( yr < 100 )
      yr = yr + 100 * Math.floor( dt.getFullYear() / 100 );

//alert( yr + '/' + mt + '/' + dy );

   /*
    * check for valid date and month values, return today if not valid:
    */

   if      ( dy < 1 || dy > 31 ) { result.stat = "dayval"; result.estr = dy; return null; }
   else if ( mt < 1 || mt > 12 ) { result.stat = "monval"; result.estr = mt; return null; }

   delete dt;

   return result.date = new Date( yr, mt - 1, dy );
}

/*
 * searchResult - search result to pass back for searchDay() and searchMonth():
 */

function searchResult()
{
   this.hits = 0;
   this.atr  = -1;
}

/*
 * searchDay - search for dayname.
 */

function searchDay( strng, result )
{
   return searchTable( strng, dayNames  , result )           // nl full daynames
       || searchTable( strng, dayAbbrs  , result )           // nl abbr daynames
       || searchTable( strng, dayNamesEn, result )           // en full daynames
       || searchTable( strng, dayAbbrsEn, result );          // en abbr daynames
}

/*
 * searchMonth - search for monthname.
 */

function searchMonth( strng, result )
{
   if ( ( hits = searchMonth2( strng, result ) ) > 0 )
      result.atr++;

   return hits;
}

function searchMonth2( strng, result )
{
   return searchTable( strng, monthNames  , result )         // nl full monthnames
       || searchTable( strng, monthAbbrs  , result )         // nl abbr monthnames
       || searchTable( strng, monthNamesEn, result )         // en full monthnames
       || searchTable( strng, monthAbbrsEn, result );        // en abbr monthnames
}

function searchTable( strng, table, result )                 // case insensitive search
{
   result.hits = 0;

   for( var i = 0; i < table.length; i++ )
   {
      if ( table[i].toLowerCase().indexOf( strng.toLowerCase() ) == 0 )
      {
         result.hits++;
         result.atr = i;
      }
   }

   return result.hits;
}


/*----------------------------------------------------------------------------
** Test - test strftime and scanDate.
**----------------------------------------------------------------------------
*/

/*
 * Test_strftime - exercise strftime().
 */

function Test_strftime()
{
   dt = new Date();     // get now

   fmts = new Array(
            "%a  abbreviated weekday name.",
            "%A  full weekday name.",
            "%b  abbreviated month name.",
            "%B  full month name.",
            "%c  local date and time representation (01-02-1996 12:34:56).",
            "%d  day of the month                   (01-31).",
            "%H  hour (24-hour clock)               (00-23).",
            "%I  hour (12-hour clock)               (01-12).",
            "%j  day of the year                   (001-366).",
            "%m  month                              (01-12).",
            "%M  minute                             (00-59).",
            "%p  local equivalent of AM or PM       (VM, NM, AM, PM).",
            "%S  second                             (00-59).",
            "%U  weeknumber of the year             (00-53),\nmonday as 1st day of week !!",
            "%w  weekday                            ( 1-7 ),\nmonday is 1, sunday is 7 !!",
            "%W  weeknumber of the year             (00-53),\nmonday as 1st day of week",
            "%x  local date representation          (01-02-1996).",
            "%X  local time representation          (12:34:56).",
            "%y  year without century               (00-99).",
            "%Y  year with century.",
            "%Z  timezone (not implemented)         ( (Timezone) ) !!",
            "%%  %" );

   document.writeln( dt.toString()  + '<br><br>\n' );

   for ( f in fmts )
   {
      var fmt = fmts[f].substr(0,2);

      document.writeln( fmt + ':' + strftime( fmt, dt ).fontcolor('blue') + '(' + fmts[f] + ')<br>\n' );
   }
}

/*
 * Test_scanDate - exercise scanDate().
 */

function Test_scanDate()
{
   fmts = new Array(
            "12",
            "12 mrt",
            "12 mar 1999",
            "feb 12",
            "feb 12 2000",
            "14 12 2002",  // 14 dec 2002
            "12 14 2002",  // 14 dec 2002
            "32 12 14",    // 14 dec 2032
            "120196",      // 12 jan 2096
            "12011996",    // 12 jan 1996
            "-1",          // yesterday
            "+1",          // tomorrow
            "za",          // zaterdag
            ".",           // today
            "",            // today
            "x",           // invalid day- or monthname
            "ma",          // ambiguous day- or monthname
            "z",           // ambiguous dayname
            "j",           // ambiguous monthname
            "32 feb",      // invalid date value
            "14 14",       // invalid month value
            "12 fb"        // invalid month value
    );

   var dt = new Date();
   document.writeln( '<br><br>' + dt.toString()  + '<br><br>\n' );

   for ( f in fmts )
   {
      var fmt = fmts[f];

      var dt = Test_scanDateMsg( fmt, fmt );

      if ( null != dt )
         document.writeln( dt.toLocaleString().fontcolor('darkorange') + '  (' + fmts[f] + ')<br>\n' );
   }
}

/*
 * Test_scanDateMsg - parse date, issue message on error.
 *
 */

function Test_scanDateMsg( strng, msg )
{
   var dateResult = new scanDateResult();

   if ( null == ( dt = scanDate( strng, dateResult ) ) )
   {
      switch( dateResult.stat )
      {
         case "daymonnon": document.writeln( msg.fontcolor('red') + ': kan de juiste dag- of maandnaam niet bepalen (' + dateResult.estr + '); corrigeer datumopgave.<br>\n' ); break;
         case "daymonamb": document.writeln( msg.fontcolor('red') + ': kan de juiste dag- of maandnaam niet bepalen (ambigu); geef meer letters op.<br>\n' ); break;
         case "dayamb"   : document.writeln( msg.fontcolor('red') + ': kan de juiste dagnaam niet bepalen (ambigu); geef meer letters op.<br>\n' ); break;
         case "monamb"   : document.writeln( msg.fontcolor('red') + ': kan de juiste maandnaam niet bepalen (ambigu); geef meer letters op.<br>\n' ); break;
         case "dayval"   : document.writeln( msg.fontcolor('red') + ': dag heeft een ongeldige waarde (' + dateResult.estr + '); corrigeer datumopgave.<br>\n' ); break;
         case "monval"   : document.writeln( msg.fontcolor('red') + ': maand heeft een ongeldige waarde(' + dateResult.estr + '); corrigeer datumopgave.<br>\n' ); break;
         default         : document.writeln( msg.fontcolor('red') + ': niet behandelde scanDate() foutcode (' + dateResult.stat + ')<br>\n' ); break;
      }
   }

   return dt;
}

/*
 * end of file
 */

