/*
 * name.js - name handling class.
 *
 * Verlof Web application
 *
 * Copyright (c) 2002, by Leiden University.
 * verlof is free software.  See the file Copying for copying permission.
 *
 * $Revision: 1.4 $
 * $Date: 2003-01-06 14:58:55+01 $
 * $Source: \\\\fwneld\\msdos\\RCS\\K\\fsc\\prj\\verlof\\src\\lib\\name.js,v $
 * %Author: M.J. Moene %
 */

/*
 * $Header: \\\\fwneld\\msdos\\RCS\\K\\fsc\\prj\\verlof\\src\\lib\\name.js,v 1.4 2003-01-06 14:58:55+01 moene Exp $
 *
 * $Log: name.js,v $
 * Revision 1.4  2003-01-06 14:58:55+01  moene
 * added mapDiacritics() in constructor;
 * added (firstname) notation as in "A.B. (Tony) Elstak".
 *
 * Revision 1.3  2002/12/23 09:42:49  moene
 * corrected namesplitting: ingeborg verhagen now becomes Verhagen-I (was Verhagen-E ing.).
 *
 * Revision 1.2  2002/09/24 08:52:56  moene
 * Corrected handling of multiple connected initials:
 * First B.C. Name now correctly becomes Name-FBC .
 *
 * Revision 1.1  2002/05/13 09:17:11  moene
 * Initial revision
 *
 */

/*
------------------------------------------------------------------------------
Name								 public class
------------------------------------------------------------------------------

Class		split a name into it's constituent parts.

Syntax		function Name( [aName: String] )

Remarks 	Name parses the name specified and tries to distinguish the
		following parts:

		title
		firstname
		initials
		prefix
		surname

		These are also the user accessible properties of a Name object.

		Name recognizes:

		[title]* [name]* [initials]* [prefix]* [surname][-[prefix]{0,2}surname]?


------------------------------------------------------------------------------
Name.toString						      public function
------------------------------------------------------------------------------

Function	format name according to the specified format.

Syntax		function Name.toString( [format: String] )

Remarks 	Name.toString returns the name formatted according to the
		specified format string.

		Ordinary characters are copied into strng. Each %c is
		replaced as follows:

		%t    title
		%f    firstname
		%i    initials (all)
		%j    initials not in firstname
		%p    prefix
		%s    surname

		The default format is: '%t %f %j %p %s'.

Arguments	format: format string.

Return value	string built.

Example 	var n = new Name( 'prof.dr. joan-janet m. van leiden-tot voorburg' );
		n.toString() yields: 'prof.dr. Joan-Janet M. van Leiden-tot Voorburg'

*/

/*----------------------------------------------------------------------------
 * class Name - parse name into parts, return sortable representation (toString())
 *----------------------------------------------------------------------------
 */

titleTable  = [ 'prof' , 'dr' , 'mr' , 'drs' , 'ir' , 'ing' , 'Baron',	];    // for input
titleTable2 = [ 'prof.', 'dr.', 'mr.', 'drs.', 'ir.', 'ing.', 'Baron',	];    // for output
prefixTable = [ 'van', 'de', 'der', 'den', 'vander', 'vanden', 'tot', ];

/*
 * Name constructor -
 *
 * Recognize: Prof.dr.ing. Joan-Janet M. van Leiden-tot Voorburg,
 * but not  : Baron Jan P. van Voorst tot Voorst-Vader.
 */

function Name( s )
{
   this.input	  = s || "";
   this.title	  =
   this.initials  =
   this.firstname =
   this.surname   =
   this.prefix	  = "";

   this.parse	  = Name_parse; 	  // unravel name into parts
   this.toString  = Name_toString;	  // return surname-title initials prefix

   this.parse();
}

/*
 * Name.parse - unravel name into various parts:
 *
 * Prof.dr.ing. Joan-Janet M. van     Leiden-tot Voorburg.
 * title	firstname     prefix  surname
 */

function Name_parse()
{
   var leftpart = "";			     // part before surname

   /*
    * strip surrounding space:
    */

   this.input = this.input.stripSpace().mapDiacritics();

   /*
    * get surname and left part
    * 'Prof.dr.ing. Joan-Janet M. van Leiden-tot Voorburg'.
    */

   var myArray;

   // 'name-pfx [pfx] name$'
   if	   ( null != (myArray = this.input.match( /([a-z_]+)-(([a-z_]+\s+){1,2})([a-z]+)$/i ) ) )
      this.surname = RegExp.$1 + '-' + RegExp.$2 + RegExp.$4;

   // 'name-name$'
   else if ( null != (myArray = this.input.match( /([a-z_]+)-([a-z_]+)$/i ) ) )
      this.surname = RegExp.$1 + '-' + RegExp.$2;

   // 'name$'
   else if ( null != (myArray = this.input.match( /([a-z]+)$/i ) ) )
      this.surname = RegExp.$1;

   // '???'
   else
   {
      this.surname = '(surname not recognized)';
      myArray = this.input.match( /(.*)/i )
   }

//   leftpart = RegExp.leftContext.stripSpace();
   leftpart = RegExp_leftContext(myArray).stripSpace();

   /*
    * extract optional (firstname) from leftpart:
    */

   if ( null != leftpart.match( /\((.*)\)/ ) )
   {
      this.firstname = RegExp.$1.toLeadCaps();
	    leftpart = leftpart.replace( /\((.*)\)/, "" )
   }

   /*
    * find and strip titles from leftpart front:
    * 'Prof.dr.ing. Joan-Janet M. van '.
    */

   var n = 0;
   var titleIndex = new Array;;

   while ( true )
   {
      var match = false;

      for ( var i  in titleTable )
      {
	 re = new RegExp( '^(' + titleTable[i] + ')[^a-z]', "i"  );

	 if ( null != (myArray = re.exec( leftpart ) ) )
	 {
	    match = true;
	    titleIndex[n++] = i;
//	      leftpart = RegExp.rightContext.stripToWord();
	    leftpart = RegExp_rightContext(myArray).stripToWord();
	 }

	 delete re;
      }

      if ( !match )
	 break;
   }

   for ( var i in titleIndex )
   {
      this.title += titleTable2[ titleIndex[i] ];
   }

   delete titleIndex;

   /*
    * find and strip prefixes from leftpart rear:
    */

   var n = 0;
   var prefixIndex = new Array;;

   leftpart = leftpart.stripSpace();		 // remove trailing space before matching

   while ( true )
   {
      var match = false;

      for ( var i in prefixTable )
      {
	 re = new RegExp( '(' + prefixTable[i] + ')$', "i"  );

	 if ( null != (myArray = re.exec( leftpart ) ) )
	 {
	    match = true;
	    prefixIndex[n++] = i;
//	      leftpart = RegExp.leftContext.stripSpace();
	    leftpart = RegExp_leftContext(myArray).stripSpace();
	 }

	 delete re;
      }

      if ( !match )
	 break;
   }

   for ( var i in prefixIndex )
   {
      this.prefix = prefixTable[ prefixIndex[i] ] + ' ' + this.prefix;
   }

   this.prefix = this.prefix.stripSpace();

   delete prefixIndex;

   /*
    * save first name, if any (may contain dashes):
    *
    * 'Joan-Janet M.'
    */

   if ( this.firstname.length <= 0 && leftpart.match( /^([\w_-]{2,})/ ) )
      this.firstname = RegExp.$1.toLeadCaps();

   /*
    * determine initials:
    */

    // not in IE:
//   this.initials = leftpart.replace( /\b(\w+)(\W*)/g, function(s, p1, p2) { p2 = ""; return p1.charAt(0).toUpperCase(); } );

   // workaround: first replace "Arno C.J." with "Arno C. J. "

   // joan-janet -> j
//   this.initials = leftpart.replace(/[.][ ]?/, ". ").replace( /((\b[a-z])([a-z._-]*[ ]?))/gi, '$2' ).toUpperCase();

   // joan-janet -> jj:
//   this.initials = leftpart.replace( /((\b[a-z])([a-z._]*[ -]?))/gi, '$2' ).toUpperCase();
//   this.initials = leftpart.replace( /\b(([a-z])([a-z._]*[ -]?))/gi, '$2' ).toUpperCase();
   this.initials = leftpart.replace(/[.][ ]?/g, ". ").replace( /\b(([a-z])([a-z._]*[ -]?))/gi, '$2' ).toUpperCase();

   /*
    * surname to lead capitals, except for prefixes:
    */

   this.surname = this.surname.toLeadCaps();
   this.surname = this.surname.toPrefixCase();
   this.prefix	= this.prefix.toPrefixCase();

   /*
    * replace underscores with spaces:
    */

   this.title	  = this.title.toSpace();
   this.initials  = this.initials.toSpace();
   this.firstname = this.firstname.toSpace();
   this.prefix	  = this.prefix.toSpace();
   this.surname   = this.surname.toSpace();
}

/*
 * Name.toString:
 *
 * 'prof.dr.ing. Joan-Janet P van Leiden-tot Voorburg'
 */

function Name_toString( fmt )
{
   var format = fmt || '%t %f %j %p %s';

   var cha;	/* current format character */
   var ndx;	/* index into formatstring */
   var len;	/* length for recursive call */

   /*
    * initialize strings:
    */

   var strng  = "";
   var tmpstr = "";

   /*
    * scan formatstring:
    */

   ndx = -1;
   while ( true )
   {
      /*
       * add the contents of tmpstr to the result:
       */

      strng  = strng + tmpstr;
      tmpstr = "";

      /*
       * characters left to scan from the formatstring?
       */

      if ( ndx < format.length ) ndx++;
      else			 break;

      cha = format.charAt( ndx );

      /*
       * check for conversion specifier:
       */

      if ( cha != "%" )
      {
	 tmpstr = cha;
	 continue;
      }

      if ( ndx >= format.length )
	 break;

      /*
       * dispatch commands:
       */

      ndx++;
      cha = format.charAt( ndx );

      switch (cha) {
	 case 't': tmpstr = this.title; break;	       // title
	 case 'f': tmpstr = this.firstname; break;     // firstname
	 case 'i': tmpstr = this.initials; break;      // initials (all)
	 case 'p': tmpstr = this.prefix; break;        // prefix
	 case 's': tmpstr = this.surname; break;       // surname

	 case 'j': var i = 0;			       // initials not in firstname
		   if ( this.initials.length  > 0     ) i++;
		   if ( this.firstname.match( /[-]/ ) ) i++;
		   tmpstr = this.initials.substr(i);
		   break;

	 default: tmpstr = cha; 		       //* %% etc.
      }
   }

   return strng.toSingleSpace().stripSpace();
}


/*----------------------------------------------------------------------------
 * RegExp - RegExp_[left|right]Context.
 *----------------------------------------------------------------------------
 */

/*
 * RegExp_[left|right]Context.
 */

function RegExp_leftContext (myArray) { return myArray.input.substring(0, myArray.index); }
function RegExp_rightContext(myArray) { return myArray.input.substring(myArray.index + myArray[0].length); }


/*----------------------------------------------------------------------------
 * Name_Test() - print parse result for several test names.
 *----------------------------------------------------------------------------
 */

function Name_test()
{
   var a = new Name( ' leiden ' );
   var b = new Name( ' j.leiden ' );
   var c = new Name( ' joan leiden ' );
   var d = new Name( ' ing. joan-janet p. Vandam ' );
   var e = new Name( ' joan leiden-van_tot_tot voorburg ' );
   var f = new Name( ' prof.dr drs joan-janet p. Van Der leiden-tot tot voorburg ' );
   var g = new Name( ' Baron Jan P. van Voorst tot Voorst-Vader ' );
   var h = new Name( ' Baron Jan P. van Voorst_tot_Voorst-Vader ' );
   var i = new Name( ' ing. Arno C.J. van Amersfoort ' );
   var j = new Name( ' ing. Arno C. J. van Amersfoort ' );
   var k = new Name( ' ing. Jan J. Jansen1 ' );
   var l = new Name( ' ingeborg verhagen ' );
   var m = new Name( ' E.C.M. (Lies) Klman ' );

   var objs = new Array(a, b, c, d, e, f, g, h, i, j, k, l, m);

   for ( o in objs )
      document.writeln( objs[o].toDebugString() + '<br>' );
}

function Name_toDebugString()
{
   return 'input     : (' + this.input	   + ')<br>'
	+ 'title     : (' + this.title	   + ')<br>'
	+ 'initials  : (' + this.initials  + ')<br>'
	+ 'firstname : (' + this.firstname + ')<br>'
	+ 'surname   : (' + this.surname   + ')<br>'
	+ 'prefix    : (' + this.prefix    + ')<br>'
	+ 'toString(): (' + this.toString()+ ')<br>';
}

Name.prototype.toDebugString = Name_toDebugString;

/*
 * end of file
 */


